# QuAbS - Quantified Abstraction Solver

QuAbS is a certifying solver for quantified Boolean formulas (QBF) in negation normal form (NNF).
It is based on a recursive counterexample guided abstraction refinement (CEGAR) algorithm.

## Solving

QuAbS accepts formulas in the standard [QCIR](http://qbf.satisfiability.org/gallery/qcir-gallery14.pdf) file format.

`$ ./quabs instance.qcir` returns eiter SAT or UNSAT


## Certification

### Format

Certificates are given as valid [AIGER](http://fmv.jku.at/aiger/) files with some additional conventions:

* The symboltable defines the original variable names from the QCIR file
* There is one additional output which must be the last output and it indicates whether the certificate is a Skolem or Herbrand certificate (value 1 and 0, respectively).

#### Example

[QCIR](http://qbf.satisfiability.org/gallery/qcir-gallery14.pdf) file encoding an equality constraint:

```
#QCIR-G14 5
forall(1)
exists(2)
output(3)
3 = and(4, 5)
4 = or(-1, 2)
5 = or(1, -2)
```

The corresponding Skolem certificate is

```
aag 1 1 0 2 0
2
2
1
i0 1
o0 2
o1 result
c
equality.cert.aag
```

### Checking Certificates

* Prerequisites: `certcheck` binary (`$ make certification`)
* Usage: `$ ./certcheck <instance.qcir> <certificate>`
* Output of `certcheck` is an AIGER file that contains exactly one (bad) output.
* The output can be verified by a SAT solver: converting using `aigtocnf` from AIGER toolset and solving the resulting DIMACS file using a standard SAT solver.
* An UNSAT result witnesses the correctness of the certificate (there is no input such that the bad output is enabled).


#### Example

`$ ./certcheck equality.qcir equality.cert.aag | aigtocnf | picosat` returns unsatisfiable, hence, the certificate is correct


## Contact

Leander Tentrup <tentrup@react-uni-saarland.de>
